const path = require('path');
const PropertiesReader = require('properties-reader');
const properties = PropertiesReader(path.join(__dirname, '..', '..', 'properties', 'object.Properties'));
const moment = require('moment');
const constantValues = require('./constants');
const gf = require('./genericFunctions');

const switchToIframe = async function(page) {
  const dropDownContainer = await page.waitForXPath(properties.get('cpq.cpq.iFrame'));
  const frameName = await page.evaluate(dropDownContainer => dropDownContainer.getAttribute('name'), dropDownContainer);
  return page.frames().find(frame => frame.name() === frameName);
};

const waitForSpinner = async function(page) {
  try {
    await page.waitForXPath(properties.get('cpq.spinner'));
    await page.waitForXPath(properties.get('cpq.spinner'), {hidden: true});
  } catch (error) {
    console.log('No Spinner displayed');
  }
};

const doubleClick = async function(page, property) {
  const element = await page.waitForXPath(property, {visible: true});
  await gf.pageWaiting(page,constantValues.TIMEOUT_2_SEC);
  await gf.clickFromPage(page, element);
  await gf.pageWaiting(page,constantValues.TIMEOUT_1_SEC);
  await gf.clickFromPage(page, element);
  console.log('Double clicked successfully');
};

const textbox =  async function(page,frame, terms, textValue) {
  const termsBox = gf.replaceXpath(properties.get('cpq.config.attributes.textbox'),terms);
  await enter(frame,termsBox,textValue);
  await gf.keyBoardPress(page);
};

const enter = async function(page, property, textValue) {
  const element = await page.waitForXPath(property);
  await element.click({ clickCount: 3 });
  await element.type(textValue);
};

const click = async function(page, property) {
  const element = await page.waitForXPath(property, {timeout: constantValues.TIMEOUT_LONG_SEC});
  await gf.clickFromPage(page, element);
};

const enterDate = async function(page, frame, property, fieldName, date) {
  const xpath = gf.replaceXpath(property, fieldName);
  const element = await frame.waitForXPath(xpath);
  frame.evaluate(el => el.value = '', element);
  await element.type(date);
  await page.keyboard('Enter');
};

const dropdown = async function(page, fieldName, value) {
  if (fieldName === 'Discount Reason') {
    await page.select(properties.get('cpq.quoteline.discount.dropdown'), value);
  } else {
    let selector = properties.get('cpq.config.attributes.dropdown');
    selector = gf.replaceXpath(selector, fieldName);
    page.select(selector, value);
  }
};

const priceBook = async function(page,fieldName) {
  const frame = await switchToIframe(page);
  await waitForSpinner(frame);
  const isHidden = await frame.$eval(properties.get('cpq.choosePriceBookSave.popup'), el => el.hasAttribute('hidden'));
  if (!isHidden) {
    await dialogBox(page,frame,fieldName);
    await waitForSpinner(frame);
  }

  return frame;
};

const dialogBox = async function cpqDialog(page,frame,label){
  const dialogBox = gf.replaceXpath(properties.get('cpq.dialogBox'),label);
  await gf.xpathClick(frame, dialogBox);
  await gf.pageWaiting(page,constantValues.TIMEOUT_1_SEC);
};

const quoteDocumentVisible = async function(page) {
  const frame = await switchToIframe(page);
  await frame.waitForSelector(properties.get('generateDocument.loading.icon'), {timeout: constantValues.TIMEOUT_LONG_SEC});
  console.log('Waited for generate document loading icon');
  await frame.waitForSelector(properties.get('generateDocument.documentOptions.Container'), {timeout: constantValues.TIMEOUT_LONG_SEC});
  console.log('Waited for generate document container to load');
  return frame;
};

const calender = async function(page, fieldName, dt, today = '') {
  const dtMoment = moment(dt, ['DD/MM/YYYY']);
  const newMonthName = dtMoment.format('MMMM');
  const todayDate = moment(new Date()).format('DD/MM/YYYY');

  await click(page, '//div[text()=\'' + fieldName + '\']//parent::div//span[@id=\'open\']');

  if (todayDate === dt || today.length > 0) {
    await click(page, '//div[text()=\'' + fieldName + '\']//parent::div//div[contains(@class,\'style-scope sb-datepicker\') and not(contains(@class,\'notCurrent\'))][text()= \'' + dtMoment.format('D') + '\']');
  } else {
    let appMonthName = await gf.getInnerText(page, '//div[text()=\'' + fieldName + '\']//parent::div//span[@id=\'monthName\']');
    while (appMonthName.toUpperCase() !== newMonthName.toUpperCase()) {
      await click(page, '//div[text()=\'' + fieldName + '\']//parent::div//span[@class=\'sb-icon-right-dir style-scope sb-datepicker\']');
      appMonthName = await gf.getInnerText(page, '//div[text()=\'' + fieldName + '\']//parent::div//span[@id=\'monthName\']');
    }

    if (fieldName === 'Start Date') {
      await page.select(properties.get('cpq.startDate'),dtMoment.format('YYYY'));
    } else
    if (fieldName === 'Expires On') {
      await page.select(properties.get('cpq.expiresOn'), dtMoment.format('YYYY'));
    }
    const finalDate = await page.waitForXPath('//div[text()=\'' + fieldName + '\']//parent::div//div[contains(@class,\'style-scope sb-datepicker\')][text()= \'' + dtMoment.format('D') + '\']');
    await gf.clickFromPage(page, finalDate);
  }
};

module.exports = {
  switchToIframe,
  waitForSpinner,
  textbox,
  doubleClick,
  enter,
  click,
  enterDate,
  dropdown,
  priceBook,
  dialogBox,
  quoteDocumentVisible,
  calender
};

