const path = require('path');
const PropertiesReader = require('properties-reader');
const properties = PropertiesReader(path.join(__dirname, '..', '..', 'properties', 'object.Properties'));
const chai = require('chai');
const assert = chai.assert;
const fs = require('fs');
const constantValues = require('./constants');

const xpathClick = async function(page, xpath) {
  await page.waitForXPath(xpath, {timeout: constantValues.TIMEOUT_MEDIUM_SEC});
  await page.evaluate(xpath => {
    const xpathResult = document.evaluate(xpath, document, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    const element = xpathResult.snapshotItem(0);
    element.click();
  }, xpath);
};

/** *********************************************************************
 Function Name: click()
 Description: This is a generic function, which will click on tabtile which you have passed
 Date:
 Author:
 Modified:
 ************************************************************************/
const click = async function(page, tabTitle) {
  const xpath_selector = '//*[@title=\'' + tabTitle + '\']';
  await xpathClick(page, xpath_selector);
  console.log('Clicked on ' + tabTitle + ' button successfully');
};

/** *********************************************************************
 Function Name: selectRecordType()
 Description: This is a generic function, which will click on radio button
 Date:
 Author:
 Modified:
 ************************************************************************/
const selectRecordType = async function(page, rectype) {
  await page.waitForSelector('.modal-container');
  const elements = await page.$$('.changeRecordTypeOptionRightColumn');

  for (const element of elements) {
    await page.evaluate((el, rectype) => {
      if (el.innerText.includes(rectype)) {
        el.click();
      }
    }, element, rectype);
  }
};

/** *********************************************************************
 Function Name: footerbutton()
 Description: This is a generic function to click on button which is on the footer for [New Account Footer Button:
 Account Creation Footer Button
 Contact Creation Footer Button
 New Opportunity and Opportunity Creation
 New Billing Information and Billing Information Creation (Not from Account)]
 Date:
 Author:
 Modified:
 ************************************************************************/
const footerButton = async function(page, buttonName) {
  const button = '//div[@class="inlineFooter"]//span[text()=\'' + buttonName + '\']/parent::button';
  await xpathClick(page, button);
  console.log('Clicked on ' + buttonName + ' footer button successfully');
};

/** ************************************************************************
 Function Name: modalfooterbutton()
 Description: This is a generic function to click on button which is on the modalfooter for [Billing Information Creation from Account
 BI Submit for Approval
 Quote Submit for Approval
 Edit Quote Payment Details
 Create Order
]
 Date:
 Author:
 Modified:
 ************************************************************************/
const modalFooterButton = async function(page, buttonName) {
  const button = '//*[contains(@class,"lds-modal__footer")]//*[text()=\'' + buttonName + '\']';
  await xpathClick(page, button);
  console.log('Clicked on ' + buttonName + ' modal footer button successfully');
};

/** *********************************************************************
 Function Name: textInput()
 Description: Generic function to enter value in testbox
 Date:
 Author:
 Modified:
 ************************************************************************/
const textInput = async function(page, label, value) {
  const input = '//*[text()=\'' + label + '\']//parent::label/following::input[1]';
  const labelInput = await page.waitForXPath(input);
  await labelInput.type(value);
};

/** *********************************************************************
 Function Name: checkBoxInput()
 Description: Generic function to check a checkbox
 Date:
 Author:
 Modified:
 ************************************************************************/
const checkBoxInput = async function(page, label) {
  const input = '//span[(text()=\'' + label + '\')]//parent::label/following::input[1]';
  const labelInput = await page.waitForXPath(input);
  await clickFromPage(page, labelInput);
};

const forceFormFooterButton = async function(page, buttonName) {
  const button = '//*[contains(@class,"slds-docked-form-footer")]//*[contains(@class,"slds-button slds-button")][text()=\'' + buttonName + '\']';
  await xpathClick(page, button);
  console.log('Clicked on ' + buttonName + ' footer button successfully');
};

/** *********************************************************************
 Function Name: fillForm()
 Description: Generic function to fill data based on the faker data
 Date:
 Author:
 Modified:
 ************************************************************************/
const fillForm = async function(page, record) {
  for (const key in record) {
    await pageWaiting(page,500);
    const func = this[key];
    if (func !== undefined && (func instanceof Function)) {
      await processRecords(page, record[key], this[key]);
    }
  }
};

const processRecords = async function(page, record, func) {
  for (const key in record) {
    if (record[key] !== null) {
      await func(page, key, record[key]);
    }
  }
};

const textArea = async function(page, label, value) {
  const textArea = '//*[text()=\'' + label + '\']//../following::textarea[1]';
  const labelArea = await page.waitForXPath(textArea);
  await labelArea.type(value);
};

const emailInput = async function(page, label, value) {
  const email = '//span[text()=\'' + label + '\']/parent::label/parent::div/input';
  const inputE = await page.waitForXPath(email);
  await inputE.type(value);
};

const dropdown = async function(page, fieldName, value) {
  const pick = '//div[@class="slds-form-element__control"]//child::span[text()=\'' + fieldName + '\']//following::div[2]//descendant::a';
  const pickList = await page.waitForXPath(pick);
  await clickFromPage(page, pickList);

  const dropDownContainerXpath = '//div[@class="slds-form-element__control"]//child::span[text()=\'' + fieldName + '\']//following::div[2]//descendant::a//parent::div/parent::div/parent::div';
  const dropDownContainer = await page.waitForXPath(dropDownContainerXpath);
  const iDAttributeValue = await page.evaluate(dropDownContainer => dropDownContainer.getAttribute('id'), dropDownContainer);

  await page.evaluate((xpath, listValue) => {
    const iterator = document.evaluate(xpath, document, null, XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);

    try {
      let thisNode = iterator.iterateNext();

      while (thisNode) {
        if (thisNode.textContent === listValue) {
          thisNode.click();
          break;
        }
        thisNode = iterator.iterateNext();
      }
    } catch (e) {
      alert('Error: Document tree modified during iteration ' + e);
    }
  }, '//div[@aria-labelledby=\'' + iDAttributeValue + '\']//ul[@class=\'scrollable\']//li/a', value);
};

const relatedTab = async function(page, relate) {
  const relateButton = '//div//descendant::li//*[text()=\'' + relate + '\']';
  await page.waitForXPath(relateButton);
  await xpathClick(page, relateButton);
};

const relatedNewContact = async function(page, newContact) {
  const newContactPath = '//div[contains(@class,\'slds-grid\')]//div[3]//a[@title=\'' + newContact + '\']';
  await xpathClick(page, newContactPath);
};

const relatedNewButton = async function(page, labelName) {
  const newButton = '//span[@title=\'' + labelName + '\']//following::div[2]/ul/li/a[@title=\'New\']';
  await xpathClick(page, newButton);
};

const search = async function(page, label, value) {
  const input = '//span[(text()=\'' + label + '\')]//parent::label/following::input[1]';
  const labelInput = await page.waitForXPath(input);
  await labelInput.type(value, {delay: 100});
  await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
  const elements = await page.$$(properties.get('generic.search.lookup.list'));
  for (const element of elements) {
    const title = await page.evaluate(el =>
      el.getAttribute('title'), element);
    if (title === value) {
      await clickFromPage(page, element);
      break;
    }
  }
};

const waitForVisualForceMessage = async function(page) {
  await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
  const elements = await page.waitForXPath(properties.get('generic.visualForceToastMessage.text'), {timeout: constantValues.TIMEOUT_4_SEC});
  const message = await page.evaluate(el => el.innerText, elements);
  await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
  await xpathClick(page, properties.get('generic.visualForceToastMessage.close.button'));
  return message;
};

const globalSearch = async function(page, searchValue, searchType) {
  let count = 0;
  while (count < constantValues.MIN_RETRY) {
    await globalSearchTypeFilter(page, searchType);
    const searchXpath = 'Search ' + searchType;
    await globalSearchValue(page, searchXpath, searchValue);
    const updatedSearchType = searchType === 'Opportunities' ? 'Opportunity' : searchType;
    if (await captureHeading(page, updatedSearchType)) {
      console.log(`Search: ${searchValue} Successful`);
      break;
    } else {
      await reload(page);
      await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
      if (await captureHeading(page, updatedSearchType)) {
        console.log(`Search: ${searchValue} Successful`);
        break;
      }
    }
    count++;
  }
  if (count === constantValues.MIN_RETRY) {
    console.log(`Initial Global Search: ${searchValue} failed`);
    await globalSearchResultsClick(page, searchValue, searchType);
  }
};

const captureHeading = async function(page, searchType) {
  try {
    const heading = await getInnerText(page, properties.get('generic.header.label'));
    return !!searchType.includes(heading);
  } catch (error) {
    return false;
  }
};

const globalSearchValue = async function(page, searchXpath, searchValue) {
  try {
    await globalEnterValue(page, searchXpath, searchValue);
  } catch (error) {
    await globalEnterValue(page, searchXpath, searchValue);
  }
  const searchElements = await page.$$(properties.get('generic.globalSearch.select'));
  for (const element of searchElements) {
    const searchTitle = await page.evaluate(el => el.getAttribute('title'), element);
    if (searchTitle === searchValue) {
      await clickFromPage(page, element);
      await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
    }
  }
  console.log('Entered search value ' + searchValue + ' and clicked on it');
};

const globalEnterValue = async function(page, searchXpath, searchValue) {
  const searchTestBox = await page.waitForXPath('//input[@title=\'' + searchXpath + '\']');
  await page.evaluate(el => el.value = '', searchTestBox);
  await searchTestBox.type(searchValue, {delay: 100});
  await page.waitForSelector('.lookup__list.visible');
  console.log(searchValue + ' entered in text box');
};

const globalSearchResultsClick = async function(page,searchValue,searchType,retryCounter = 0) {
  try {
    await globalSearchResults(page, searchValue, searchType);
    await globalSearchClick(page);
    console.log(`SearchClick : ${searchValue} was successful`);
  } catch (error) {
    if (retryCounter < constantValues.MAX_RETRY) {
      retryCounter++;
      console.log('Retrying globalSearchResultClick ' + retryCounter);
      await pageWaiting(page,constantValues.TIMEOUT_4_SEC);
      await globalSearchResultsClick(page, searchValue, searchType, retryCounter);
    } else {
      console.log(error);
      assert.fail(`Search: ${searchValue} failed`);
    }
  }
};

const globalSearchResults = async function(page, searchValue, searchType) {
  await globalSearchTypeFilter(page, searchType);
  const searchXpath = 'Search ' + searchType;
  const searchTestBox = await page.waitForXPath('//input[@title=\'' + searchXpath + '\']');
  await page.evaluate(el => el.value = '', searchTestBox);
  await searchTestBox.type(searchValue);
  await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
  await page.keyboard.press('Enter');
};

const globalSearchTypeFilter = async function(page, searchType) {
  await xpathClick(page, properties.get('generic.globalSearch.filter'));
  const elements = await replaceXpath(properties.get('generic.globalSearch.filterType'),searchType);
  await xpathClick(page,elements);
  console.log('Filter type ' + searchType + ' selected successfully');
};

const globalSearchClick = async function(page){
  try {
    await pageWaiting(page,constantValues.TIMEOUT_4_SEC);
    await page.waitForSelector('.scroller.actionBarPlugin',{visible: true});
    const regName = await page.waitForXPath(properties.get('generic.global.Search.Click'));
    await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
    await page.evaluate(el => el.click(), regName);
  } catch (error){
    await page.reload();
    await pageWaiting(page,constantValues.TIMEOUT_1_SEC);
    await globalSearchClick(page);
  }
};

const getInnerText = async function(page, xpath) {
  const headerContainer = await page.waitForXPath(xpath, {hidden: false});
  return await page.evaluate(headerContainer => headerContainer.innerText, headerContainer);
};

const lookUpLink = async function(page, labelName, value) {
  const lookup = '//span[text()=\'' + labelName + '\']//parent::div[contains(@class,\'test-id__field-label-container\')]//following-sibling::div//a[text()=\'' + value + '\']';
  await xpathClick(page, lookup);
};

const screenShot = async function(page, filename = '') {
  const dir = './screenShots';
  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir);
  }
  if (filename.length > 0) {
    await page.screenshot({
      path: 'screenShots/errorScreenShot-' + filename + '-' + Date() + '.png',
      fullPage: true
    });
  } else {
    await page.screenshot({path: 'screenShots/errorScreenShot-' + Date() + '.png', fullPage: true});
  }
};

const modalButton = async function(page) {
  await xpathClick(page, properties.get('generic.model.close.button'));
};

const reload = async function(page) {
  await page.reload();
  await page.waitForNavigation({timeout: constantValues.TIMEOUT_LONG_SEC, waitUntil: 'load'});
};

const isVisible = async function(page, selector) {
  await pageWaiting(page,constantValues.TIMEOUT_2_SEC);
  return await page.$$eval(selector, e1 => e1.length > 0);
};

const getDetailsPageFieldValues = async function(page, fieldName, property) {
  const xpath = replaceXpath(property, fieldName);
  return getInnerText(page, xpath);
};

const switchToNewPage = async function(browser) {
  const pages = await browser.pages();
  return pages[1];
};

const switchToMainPage = async function(browser) {
  const pages = await browser.pages();
  return pages[0];
};

const replaceXpath = function(elementProperty, valueToBeReplaced) {
  return elementProperty.replace('propertyToBeReplaced', valueToBeReplaced);
};

const scrollDown = async function(page) {
  page.evaluate(() => {
    window.scrollBy(0, window.innerHeight);
  });
};

const scrollUp = async function(page){
  page.evaluate(() => {
    window.scrollBy(0, -window.innerHeight);
  });
};

const clickFromPage = async function(page, element) {
  await page.evaluate(el => el.click(), element);
};

const formatAmount = function(amount) {
  let formatAmount;
  if (amount.includes('(')) {
    formatAmount = amount.split('(');
    return formatAmount[0].trim();
  } else {
    formatAmount = amount;
    return formatAmount.trim();
  }
};

const keyBoardPress = async function(page) {
  await page.keyboard.press('Tab', {delay: 10});
};

const quoteButtons = async function(page, buttonName) {
  try {
    await pageWaiting(page,constantValues.TIMEOUT_4_SEC);
    const quoteButton = '//*[text()=\'' + buttonName + '\']';
    await xpathClick(page, quoteButton);
    await pageWaiting(page,constantValues.TIMEOUT_6_SEC);
  } catch (error) {
    console.log(error);
  }
};

const submitForApproval = async function(page, buttonName,footerClick = false) {
  try {
    await quoteButtons(page,buttonName);
    if (footerClick){
      await textArea(page, 'Comments', 'Please approve');
      await modalFooterButton(page, buttonName);
    }
  } catch (error) {
    console.log(error);
  }
};

const buttonClick = async function(page, property, textValue) {
  const xpath = replaceXpath(property, textValue);
  const element = await page.waitForXPath(xpath, {visible: true});
  await clickFromPage(page, element);
  console.log('Clicked on ' + textValue + ' button successfully');
};

const selectDropdown = async function(page, fieldName, value) {
  if (fieldName === 'Discount Reason') {
    await page.select(properties.get('cpq.quoteline.discount.dropdown'), value);
  } else {
    let selector = properties.get('cpq.config.attributes.dropdown');
    selector = replaceXpath(selector, fieldName);
    page.select(selector, value);
  }
};

const select_formControl = async function(page,fieldName,value){
  let selectorForm = properties.get('generic.select_formControl');
  selectorForm = replaceXpath(selectorForm, fieldName);
  page.select(selectorForm, value);
};

const labelInput = async function(page, label, value) {
  const input = '//label[(text()=\'' + label + '\')]//parent::label/following::input[1]';
  const labelInput = await page.waitForXPath(input);
  await labelInput.type(value);
};

const comboBox = async function(page, fieldName, value) {
  const pick = '//div[@class="slds-form-element__control"]//parent::lightning-combobox//child::label[text()=\'' + fieldName + '\']//following::input[1]';
  const pickList = await page.waitForXPath(pick);
  await clickFromPage(page, pickList);

  await page.evaluate((xpath, listValue) => {
    const iterator = document.evaluate(xpath, document, null, XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
    console.log(`Iterator Xpath  : ${iterator}`);

    try {
      let thisNode = iterator.iterateNext();

      while (thisNode) {
        if (thisNode.textContent === listValue) {
          thisNode.click();
          break;
        }
        thisNode = iterator.iterateNext();
      }
    } catch (e) {
      alert('Error: Document tree modified during iteration ' + e);
    }
  }, '//div[@class="slds-form-element__control"]//parent::lightning-combobox//child::label[text()=\'' + fieldName + '\']//following::input[1]//following::div[2]//child::lightning-base-combobox-item/span[2]//span', value);
};

const pageWaiting = async function(page, time) {
  await page.waitForTimeout(time);
};


module.exports = {
  labelInput,
  selectDropdown,
  select_formControl,
  xpathClick,
  click,
  selectRecordType,
  footerButton,
  modalFooterButton,
  textInput,
  checkBoxInput,
  forceFormFooterButton,
  fillForm,
  textArea,
  emailInput,
  dropdown,
  relatedTab,
  relatedNewContact,
  relatedNewButton,
  search,
  waitForVisualForceMessage,
  globalSearch,
  captureHeading,
  getInnerText,
  lookUpLink,
  screenShot,
  modalButton,
  reload,
  isVisible,
  getDetailsPageFieldValues,
  switchToNewPage,
  switchToMainPage,
  replaceXpath,
  scrollDown,
  scrollUp,
  clickFromPage,
  formatAmount,
  keyBoardPress,
  submitForApproval,
  buttonClick,
  globalSearchResults,
  globalSearchClick,
  globalSearchResultsClick,
  quoteButtons,
  comboBox,
  pageWaiting
};
