const puppeteer = require('puppeteer');
const jsforce = require('jsforce');

/** *********************************************************************
 Function Name: loginToSalesforce()
 Description: Login into salesforce by entering username and password
 Date:
 Author:
 Modified:
 ************************************************************************/
const loginToSalesforce = async function(browser, url, userName, password) {
  const page = await browser.newPage();
  try {
    await page.setViewport({width: 1920, height: 1080});
    page.setDefaultNavigationTimeout(200000);
    await page.goto(url);
    await page.type('#username', userName);
    await page.type('#password', password);
    await Promise.all([
      page.waitForNavigation({waitUntil: 'load'}),
      page.waitForNavigation({waitUntil: 'domcontentloaded'}),
      page.evaluate(() => {
        document.querySelector('#Login').click();
      }),
    ]);
    console.log('Successfully logged into salesforce as ' + userName + '\n');

    return page;
  } catch (e) {
    console.log(e);
  }
};

/** *********************************************************************
 Function Name: launchChrome()
 Description: This function will launch chrome browser and return browser
 Date:
 Author:
 Modified:
 ************************************************************************/
const launchChrome = async function(headless, executablePath = null) {
  try {
    return await puppeteer.launch(
        {
          headless: JSON.parse(headless),
          executablePath,
          args: ['--disable-notifications', '--no-sandbox', '--disable-gpu', '--disable-dev-shm-usage', '--start-maximized', '--start-fullscreen','--disable-features=site-per-process']
        });

  } catch (e) {
    console.log(e);
  }
};

/** *********************************************************************
 Function Name: loginToJSForce()
 Description: Using jsforce login into salesforce by entering username and password
 Date:
 Author:
 Modified:
 ************************************************************************/
const loginToJSForce = async function(url, userName, password) {
  try {
    const conn = new jsforce.Connection({ loginUrl: url });
    await conn.login(userName, password);
    console.log('Successfully logged into salesforce api as ' + userName + '\n');
    return conn;
  } catch (error) {
    console.log('jsforce login failed');
    console.log(error);
  }
};

const loginToSecurFrontDoor = async function(browser, conn) {
  const page = await browser.newPage();
  try {
    await page.setViewport({width: 1920, height: 1080});
    page.setDefaultNavigationTimeout(200000);
    const pageURL = `${conn.instanceUrl}/secur/frontdoor.jsp?sid=${conn.accessToken}`;
    await page.goto(pageURL);
    console.log('Successfully logged into salesforce');
    return page;
  } catch (error) {
    console.log(error);
  }
};

const salesforceRecordPage = async function(page,conn,recordName,recordId){
  try {
    const recordURL = `${conn.instanceUrl}/lightning/r/${recordName}/${recordId}/view`;
    await page.goto(recordURL);
    return recordURL;
  } catch (error) {
    console.log(error);
  }
};

module.exports = {
  loginToSalesforce,
  launchChrome,
  loginToJSForce,
  loginToSecurFrontDoor,
  salesforceRecordPage
};

